# Let's generate 1000 samples from a normal distribution
# with mean 2 and variance 4
set.seed(1342)
y<-rnorm(1000, 2, 2)


# Suppose mu and sigma2 are unknowns
# We only ``know'' observations y 

# Gibbs sampler

# Let's define the hyperparameters
# Assume we do not have information, so we use vague priors
	
      # Hyperparameters for f(mu)
	sigma2.mu<-100
	m<-0

	# Hyperparameters for f(sigma2)
	alpha<-0.1
      beta<-0.1

# Let's sart with running the Gibbs sampler for 10 iterations (S=10)
it<-10

# The matrices "mu" y "sigma2" will save the values sampled
# at each iteration of the algorithm
# The first value of each matrix is the initial value
# Starting values: mu = 0,  sigma2=1
mu<-matrix(0,it+1)
sigma2<-matrix(1,it+1)



for (i in 2:(it+1)) { 
sigma2[i]<-1/rgamma(1, length(y)/2+alpha, sum((y-mu[i-1])^2)/2+beta)

mu[i]<-rnorm(1, (mean(y)*length(y)*sigma2.mu+m*sigma2[i])/(length(y)*sigma2.mu+sigma2[i]),
		sqrt((sigma2[i]*sigma2.mu)/(length(y)*sigma2.mu+sigma2[i])))

print(c("iteration = ",i-1))
}

mean(mu)
mean(sigma2)

# Let's compare the sampled values against the ``real'' values (mu = 2, sigma2=4)

var(mu)
var(sigma2)


plot(mu, type='l', xlab="", 
	ylab="", main="Sampleo de Gibbs 
		n=1000, 10 iteraciones")
mtext(side=2, line=2.5, expression(mu[s]), cex=1.2)
mtext(side=1, line=2.5, "Iteraciones del Algoritmo, s")


plot(sigma2, type='l', xlab="", 
	ylab="", main="Gibbs sampler
		n=1000, s=10")
mtext(side=2, line=2.5, expression(sigma^2[s]), cex=1.2)
mtext(side=1, line=2.5, "Iterations (s)")


# Now try with 1,000 iterations
# (same hyperparameters, starting values, etc.)

it<-1000

mu<-matrix(0,it+1)
sigma2<-matrix(1,it+1)


for (i in 2:(it+1)) { 
sigma2[i]<-1/rgamma(1, length(y)/2+alpha, sum((y-mu[i-1])^2)/2+beta)

mu[i]<-rnorm(1, (mean(y)*length(y)*sigma2.mu+m*sigma2[i])/(length(y)*sigma2.mu+sigma2[i]),
		sqrt((sigma2[i]*sigma2.mu)/(length(y)*sigma2.mu+sigma2[i])))

print(c("iteration = ",i-1))
}


mean(mu)
mean(sigma2)

#  Posterior means are closer to the ``real'' values
# Plus, variation across iterations diminishes
var(mu)
var(sigma2)


plot(mu, type='l', xlab="", 
	ylab="", main="Gibbs sampler
		n=1000, s=1000")
mtext(side=2, line=2.5, expression(mu[s]), cex=1.2)
mtext(side=1, line=2.5, "Iterations(s)")

plot(sigma2, type='l', xlab="", 
	ylab="",  main="Gibbs sampling
		n=1000, s=1000")
mtext(side=2, line=2.5, expression(sigma^2[s]), cex=1.2)
mtext(side=1, line=2.5, "Iterations (s)")


# Si sigo aumentando it, la "calidad de la estimacion" mejora

# Por supuesto, si ademas de aumentar las iteraciones tengo mas datos, es mejor

set.seed(1342)
y<-rnorm(1000000, 2, 2)

it<-5000

mu<-matrix(0,it+1)
sigma2<-matrix(1,it+1)


for (i in 2:(it+1)) { 
sigma2[i]<-1/rgamma(1, length(y)/2+alpha, sum((y-mu[i-1])^2)/2+beta)

mu[i]<-rnorm(1, (mean(y)*length(y)*sigma2.mu+m*sigma2[i])/(length(y)*sigma2.mu+sigma2[i]),
		sqrt((sigma2[i]*sigma2.mu)/(length(y)*sigma2.mu+sigma2[i])))

print(c("iteracion = ",i-1))
}


mean(mu)
mean(sigma2)


plot(mu, type='l', xlab="", 
	ylab="", main="Sampleo de Gibbs 
		n=1000000, 5000 iteraciones")
mtext(side=2, line=2.5, expression(mu[s]), cex=1.2)
mtext(side=1, line=2.5, "Iteraciones del Algoritmo, s")

plot(sigma2, type='l', xlab="", 
	ylab="",  main="Sampleo de Gibbs 
		n=1000000, 5000 iteraciones")
mtext(side=2, line=2.5, expression(sigma^2[s]), cex=1.2)
mtext(side=1, line=2.5, "Iteraciones del Algoritmo, s")

