library(coda)
# Generate 1000 values from a normal distribution
# with mean=2 and variance=4
set.seed(1342)
y<-rnorm(1000, 2, 2)


  # Hyperparmaeters for f(mu)
	sigma2.mu<-100
	m<-0

# Hyperparameters for f(sigma2)
	alpha<-0.1
      beta<-0.1

# Let's run the Gibbs sampler for 10 iterations
it<-10

mu<-matrix(0,it+1)
sigma2<-matrix(1,it+1)



for (i in 2:(it+1)) { 

sigma2[i]<-1/rgamma(1, length(y)/2+alpha, sum((y-mu[i-1])^2)/2+beta)

mu[i]<-rnorm(1, (mean(y)*length(y)*sigma2.mu+m*sigma2[i])/(length(y)*sigma2.mu+sigma2[i]),
		sqrt((sigma2[i]*sigma2.mu)/(length(y)*sigma2.mu+sigma2[i])))

print(c("iteration = ",i-1))
}

# Check convergence using Geweke's criterion
geweke.diag(mu)

geweke.diag(sigma2)

# None of the parameters converged

# Let's try running the chain for 1000 iterations
it<-1000

mu<-matrix(0,it+1)
sigma2<-matrix(1,it+1)



for (i in 2:(it+1)) { 

sigma2[i]<-1/rgamma(1, length(y)/2+alpha, sum((y-mu[i-1])^2)/2+beta)

mu[i]<-rnorm(1, (mean(y)*length(y)*sigma2.mu+m*sigma2[i])/(length(y)*sigma2.mu+sigma2[i]),
		sqrt((sigma2[i]*sigma2.mu)/(length(y)*sigma2.mu+sigma2[i])))

print(c("iteration = ",i-1))
}


geweke.diag(mu) # Converged!
geweke.diag(sigma2) # Not yet!


# Let's try with 5000 iterations
it<-5000

mu<-matrix(0,it+1)
sigma2<-matrix(1,it+1)



for (i in 2:(it+1)) { 

sigma2[i]<-1/rgamma(1, length(y)/2+alpha, sum((y-mu[i-1])^2)/2+beta)

mu[i]<-rnorm(1, (mean(y)*length(y)*sigma2.mu+m*sigma2[i])/(length(y)*sigma2.mu+sigma2[i]),
		sqrt((sigma2[i]*sigma2.mu)/(length(y)*sigma2.mu+sigma2[i])))

print(c("iteration = ",i-1))
}

# Checking convergence
geweke.diag(mu) 

geweke.diag(sigma2) 

# Both converged!


# Let's try with Heidel's criterion

heidel.diag(mu)

heidel.diag(sigma2)






