setwd("C:/Users/gk249/Downloads")
library(mvtnorm)
df<-read.csv("Logit.csv")

country<-as.numeric(factor(df$country.id), levels=unique(df$country.id))
num.country<-length(unique(country))

# 1= survive, 2=merge, 3=dissolve
y<-df$dependent.variable

X<-cbind(1,  df$covariate.1, df$covariate.2)
colnames(X)<-c("Intercept", "Covariate 1", "Covariate 2")

n<-nrow(X)

Matrix.Country<-matrix(0,n,num.country)
for (i in 1:num.country) Matrix.Country[country==i,i]<-1


# Iterations &  burn-in
 nsim<-1000				# Number of MCMC Iterations
 thin<-10                          # Thinning Interval (should be a divisor of nsim/2)
 burn<-200                       # Burn-in
 lastit<-(nsim-burn)/thin	      # Last stored value

# Priors
set.seed(26135)
mu0<-rep(0,ncol(X))
Tau0<-diag(1,ncol(X))
Sigma0<-solve(Tau0)

# Store Values
 Beta<-array(0,c(lastit,ncol(X),3)) 
 Sigma.country<-array(0, c(lastit,1,3))
 Reffect.country<-array(0,c(lastit,num.country,3))



for (ch in 1:3) {

# Initial values for each chain
  beta<-rnorm(ncol(X))
  cov2<-100*diag(ncol(X))
  reffect.country<-rnorm(num.country) 
  sigma.country<-runif(1)
 

for (i in 1:nsim) {


# Update beta using MH algorithm
 # Calculate Likelihood based on old value of beta
  eta<-X%*%beta+reffect.country[country]

  p1<-1/(1+exp(eta))
  p2<-p1*exp(eta)

  lold<-sum(log(p1)*(y==0)+log(p2)*(y==1))

 # Draw Candidates
  beta.candidate<-rmvnorm(1, beta, .005*cov2)  # Tuning parameter to reach appropriate acceptance rate	

 eta.new<-X%*%c(beta.candidate)+reffect.country[country]
  p1.new<-1/(1+exp(eta.new))
  p2.new<-p1.new*exp(eta.new)

  lnew<-sum(log(p1.new)*(y==0)+log(p2.new)*(y==1))

# Acceptance prob on log scale
   ratio<-lnew+dmvnorm(beta.candidate,mu0,Sigma0,log=T)-	
	(lold+dmvnorm(beta,mu0,Sigma0,log=T))
   if(!is.na(ratio) & log(runif(1))<ratio){
    beta<-c(beta.candidate)
    }


###################################
# Update Country reffects using MH #
 ##################################
 reffect.country.new<-rnorm(num.country, reffect.country, 0.0001)
  eta.country.old<-X%*%beta+reffect.country[country]
 
  p1.country.old<-1/(1+exp(eta.country.old))
  p2.country.old<-p1.country.old*exp(eta.country.old)

 l.country.old<-log(p1.country.old)*(y==0)+log(p2.country.old)*(y==2)



 eta.country.new<-X%*%beta+reffect.country.new[country]
  p1.country.new<-1/(1+exp(eta.country.new))
  p2.country.new<-p1.country.new*exp(eta.country.new)

 l.country.new<-log(p1.country.new)*(y==0)+log(p2.country.new)*(y==1)

 ratio.country<-(t(Matrix.Country)%*%(l.country.new-l.country.old)) + 
	dnorm(reffect.country.new,0,sigma.country, log=TRUE)-
	dnorm(reffect.country,0,sigma.country, log=TRUE)

 if (!is.na(sum(ratio.country))) {
u<-(log(runif(num.country))<  ratio.country)*1
  reffect.country[!is.na(u) & u==1]<-reffect.country.new[!is.na(u) & u==1]
}
 ########################
 # Update sigma.country #
 #########################
sigma.country<-1/rgamma(1,num.country/2+0.1, 0.1+crossprod(reffect.country)/2)
    
# Store Results
  if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
   Beta[j,,ch]<-beta
   Sigma.country[j,,ch]<-c(sigma.country)
   Reffect.country[j,,ch]<-as.vector(reffect.country)
 }


if (i%%1000==0)  print(c(ch, i))

}
}

 
# End of MCMC algorithm

## Convergence checks
library(coda)
BETA<-mcmc.list(as.mcmc(Beta[,,1]), as.mcmc(Beta[,,2]), 
		as.mcmc(Beta[,,3]))
SIGMA.COUNTRY<-mcmc.list(as.mcmc(Sigma.country[,,1]), 
as.mcmc(Sigma.country[,,2]), as.mcmc(Sigma.country[,,3]))


gelman.diag(BETA)
gelman.diag(SIGMA.COUNTRY)

# Need longer chains!

