setwd("C:/Users/gk249/Downloads")
library(mvtnorm)
df<-read.csv("Logit.csv")

country<-as.numeric(factor(df$country.id), levels=unique(df$country.id))
num.country<-length(unique(country))
n.country<-table(country)

# 1= survive, 2=merge, 3=dissolve
y<-df$dependent.variable

X<-cbind(1,  df$covariate.1, df$covariate.2)
colnames(X)<-c("Intercept", "Covariate 1", "Covariate 2")

n<-nrow(X)
Matrix.Country<-matrix(0,n,num.country)
for (i in 1:num.country) Matrix.Country[country==i,i]<-1


#### Priors 
 mbeta0<-rep(0,ncol(X))				# Prior Mean for Beta
 vbeta0<-diag(100,ncol(X))			# Prior Cov of Beta (vague)
 prec0<-solve(vbeta0)			# Prec beta
 vbeta<-solve(prec0+crossprod(X,X))	# Posterior variance beta
 l0<-d0<-.1					# Hyperparameters for precb


# MCMC Info #
#############
 nsim<-1000 # MCMC Iterations
 thin<-10         	# Thinning Interval 
 burn<-200   		 # Burn-in
 lastit<-(nsim-burn)/thin	# Last stored value


# Store Values
 Beta<-array(0,c(lastit,ncol(X),3)) 
 Sigma.country<-array(0, c(lastit,1,3))
 Reffect.country<-array(0,c(lastit,num.country,3))


###################
# Gibbs sampler ####

###################
for (ch in 1:3) {

# Initial values for each chain
  beta<-rnorm(ncol(X))
  cov2<-100*diag(ncol(X))
  reffect.country<-rnorm(num.country) 
  sigma.country<-runif(1)
 precb.country<-1/sigma.country
  z<-rep(0,n)				# Latent Normal Variable


for (i in 1:nsim) {
  muz<-X%*%beta+reffect.country[country]
  z[y==0]<-qnorm(runif(n,0,pnorm(0,muz,1)),muz,1)[y==0]
  z[y==1]<-qnorm(runif(n,pnorm(0,muz,1),1),muz,1)[y==1]
   mbeta <- vbeta%*%(prec0%*%mbeta0+crossprod(X,z-reffect.country[country]))
   beta<-c(rmvnorm(1,mbeta,vbeta))
 
m.country<-as.vector(t(Matrix.Country)%*%(z-X%*%beta))  # Requires R 64-bit
reffect.country<-rnorm(num.country,(1/(precb.country+n.country))*m.country,sqrt(1/(precb.country+n.country)))
precb.country<-rgamma(1,l0+num.country/2,d0+crossprod(reffect.country)/2)
sigma.country<-1/precb.country

   if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
   Beta[j,,ch]<-beta
   Sigma.country[j,,ch]<-c(sigma.country)
   Reffect.country[j,,ch]<-as.vector(reffect.country)
}

if (i%%1000==0)  print(c(ch, i))

}
}

 
# End of MCMC algorithm (Gibbs sampler)

## Convergence checks
library(coda)
BETA<-mcmc.list(as.mcmc(Beta[,,1]), as.mcmc(Beta[,,2]), 
		as.mcmc(Beta[,,3]))
SIGMA.COUNTRY<-mcmc.list(as.mcmc(Sigma.country[,,1]), 
as.mcmc(Sigma.country[,,2]), as.mcmc(Sigma.country[,,3]))
REFFECT.COUNTRY<-mcmc.list(as.mcmc(Reffect.country[,,1]), as.mcmc(Reffect.country[,,2]), as.mcmc(Reffect.country[,,3]))

gelman.diag(BETA)
gelman.diag(SIGMA.COUNTRY)
gelman.diag(REFFECT.COUNTRY)

# But note that
gelman.diag(REFFECT.COUNTRY)[[1]][,1]<1.2