mcmc_logitR<-function(y=y, X=X, chains=chains, nsim=nsim, thin=thin, burn=burn){

lastit<-(nsim-burn)/thin	      # Last stored value

# Priors
mu0<-rep(0,ncol(X))
Tau0<-diag(1,ncol(X))
Sigma0<-solve(Tau0)

# Store Values
 Beta<-array(0,c(lastit,ncol(X),chains)) 


for (ch in 1:chains) {

# Initial values for each chain
  beta<-rnorm(ncol(X))
  cov2<-100*diag(ncol(X))
 
 

for (i in 1:nsim) {


# Update beta using MH algorithm
 # Calculate Likelihood based on old value of beta
  eta<-X%*%beta

  p1<-1/(1+exp(eta))
  p2<-p1*exp(eta)

  lold<-sum(log(p1)*(y==0)+log(p2)*(y==1))

 # Draw Candidates
  beta.candidate<-rmvnorm(1, beta, .005*cov2)  # Tuning parameter to reach appropriate acceptance rate	

 eta.new<-X%*%c(beta.candidate)
  p1.new<-1/(1+exp(eta.new))
  p2.new<-p1.new*exp(eta.new)

  lnew<-sum(log(p1.new)*(y==0)+log(p2.new)*(y==1))

# Acceptance prob on log scale
   ratio<-lnew+dmvnorm(beta.candidate,mu0,Sigma0,log=T)-	
	(lold+dmvnorm(beta,mu0,Sigma0,log=T))
   if(!is.na(ratio) & log(runif(1))<ratio){
    beta<-c(beta.candidate)
    }

# Store Results
  if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
   Beta[j,,ch]<-beta
 }


}
}

return("Beta"=Beta)

} 
