#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]

using namespace Rcpp;
using namespace arma;


//[[Rcpp:plugins(cpp11)]]
#include <omp.h>
//[[Rcpp:plugins(openmp)]]

const double log2pi = std::log(2.0 * M_PI);


arma::mat mvrnormArma(int n, arma::mat mu, arma::mat sigma) {
          int ncols = sigma.n_cols;
          arma::mat Y = arma::randn(n, ncols);
     arma::mat Z=arma::repmat(mu, 1, n).t() + Y * arma::chol(sigma);
     
     return (Z.t());
 }


double am_mult2(const arma::mat& rhs)
{
  arma::mat ip= rhs.t();
 return (ip(0));
}



arma::vec dmvnrm_arma(arma::mat x,  
                      arma::rowvec mean,  
                      arma::mat sigma, 
                      bool logd = false) { 
    int n = x.n_rows;
    int xdim = x.n_cols;
    arma::vec out(n);
    arma::mat rooti = arma::trans(arma::inv(trimatu(arma::chol(sigma))));
    double rootisum = arma::sum(log(rooti.diag()));
    double constants = -(static_cast<double>(xdim)/2.0) * log2pi;
    
    for (int i=0; i < n; ++i) {
        arma::vec z = rooti * arma::trans( x.row(i) - mean) ;    
        out(i)      = constants - 0.5 * arma::sum(z%z) + rootisum;     
    }  
      
    if (logd == false) {
        out = exp(out);
    }
    return(out);
}



double vm_convert(const arma::mat& rhs)
{
  arma::mat ip= rhs;
 return (ip(0));
}



// [[Rcpp::export()]]
Rcpp::List mcmc_logit_HPC(arma::mat y,
arma::mat X,
arma::mat I_outcome,
arma::mat Cov_Beta,
arma::mat betastart,
arma::rowvec densitybeta,
int mcmc = 100,
int burn=10,
int thin=10,
int chains=3
 ) {

 // Dimensions
int r= X.n_cols ;
int N=X.n_rows;



 // Current Containers

  arma::mat beta = betastart ;
  arma::mat betacandidate(r,1);
  betacandidate.fill(0.0);
 
 arma::mat etaold_beta(N, 2) ;
 etaold_beta.fill(0.0) ;
 arma::mat etanew_beta(N, 2) ;
 etanew_beta.fill(0.0) ;

arma::mat sigmabetadensity(r,r,arma::fill::eye);


 int lastit= (mcmc-burn)/thin	;

// Trace Containers
  arma::cube tracebeta(r, lastit,chains) ;



omp_set_num_threads(chains);

#pragma omp parallel for
for (int chain = 0; chain < chains; ++chain) {

for (int iter = 0 ; iter < mcmc ; ++iter) {


etaold_beta.col(0)=1/(1+exp(X*beta));
etaold_beta.col(1)=exp(X*beta)%etaold_beta.col(0);

betacandidate=mvrnormArma(1,  beta, Cov_Beta);

etanew_beta.col(0)=1/(1+exp(X*betacandidate));
etanew_beta.col(1)=exp(X*betacandidate)%etanew_beta.col(0);


double accept_beta=vm_convert((sum(I_outcome.col(0)%log(etanew_beta.col(0)),0)+sum(I_outcome.col(1)%log(etanew_beta.col(1)),0))-
			(sum(I_outcome.col(0)%log(etaold_beta.col(0)),0)+sum(I_outcome.col(1)%log(etaold_beta.col(1)),0)))+
am_mult2(dmvnrm_arma(betacandidate.t(),densitybeta, sigmabetadensity, true))-am_mult2(dmvnrm_arma(beta.t(),densitybeta, sigmabetadensity, true));


double unibeta=log(Rf_runif(0,1));

if(unibeta<accept_beta) {
for (int l=0; l<r; ++l) {
beta(l)=betacandidate(l);
}
}



 // TRACE
if ((iter+1)> burn & (iter+1)%thin==0) {
int j=((iter)-burn)/thin;
tracebeta.subcube(0, j, chain, (r-1), j, chain)=beta;

} //end trace


}  // end iterations
} // end chains

// Returns
Rcpp::List ret;
 ret["beta"] = tracebeta ;


 return(ret) ;
 

}

