# Generate data
library(MASS)
set.seed(123)
rho <- -0.6 # ``real'' value of rho = -0.6
mu1 <- 0; sigma1 <- 1
mu2 <- 0; sigma2 <- 1


Data <- mvrnorm(10000, mu = c(0,0), 
		Sigma =  matrix(c(sigma1^2, sigma1*sigma2*rho, sigma1*sigma2*rho, sigma2^2),2))

x<-Data[,1]
y<-Data[,2]
n<-length(x)

# Assume the only unknown is rho

# Create a function to evaluate the 
# posterior f(rho|Data)

lnpost<-function(rho){
-((n)/2)*log(1-rho^2)- 0.5* (sum(x^2)-2*rho*sum(x*y)+sum(y^2))/(1-rho^2)
}


# iterations
it<-10000

# Initial Values
corr<-matrix(0, it)

# I want to see the % of accepted proposals
accept.total<-0

for (i in 2:(it+1)) {

candidate<-corr[i-1]+runif(1, -0.1, 0.1)

# rho cannot exceed 1 in abs. value 
if (abs(candidate)>= 1) {
accept<-0
corr[i]<-corr[i-1]
} else { # if -1 < rho < 1

# Draw uniform 0 - 1 
u<-runif(1,0,1)

# Ratio (en logaritmos)
logRatio <- lnpost(candidate)-lnpost(corr[i-1])

if (logRatio < log(u) ) {
accept<-0
corr[i]<-corr[i-1]
} else {
corr[i]<-candidate
accept<-1
}

}

# Total de valores aceptados
accept.total<-accept.total+accept

print(paste("iteration = ", i-1)) 
}

mean(corr)

var(corr)

plot(corr, type='l', 
xlab="", ylab="", ylim=c(-0.7, 0.1), main=expression(paste(rho, " samples")))
mtext(side=2, line=2.5, expression(rho^s), cex=1.2)
mtext(side=1, line=2.5, "Iterations (s)")

