setwd("C:/Users/gk249/Downloads")
library(mvtnorm)
df<-read.csv("Logit.csv")

# 1= survive, 2=merge, 3=dissolve
y<-df$dependent.variable

X<-cbind(1,  df$covariate.1, df$covariate.2)
colnames(X)<-c("Intercept", "Covariate 1", "Covariate 2")

n<-nrow(X)

#### Priors 
 mbeta0<-rep(0,ncol(X))				# Prior Mean for Beta
 vbeta0<-diag(100,ncol(X))			# Prior Cov of Beta (vague)
 prec0<-solve(vbeta0)			# Prec beta
 vbeta<-solve(prec0+crossprod(X,X))	# Posterior variance beta

# MCMC Info #
#############
 nsim<-1000 # MCMC Iterations
 thin<-10         	# Thinning Interval 
 burn<-200   		 # Burn-in
 lastit<-(nsim-burn)/thin	# Last stored value


# Store Values
 Beta<-array(0,c(lastit,ncol(X),3)) 

###################
# Gibbs sampler ####

###################
for (ch in 1:3) {

# Initial values for each chain
  beta<-rnorm(ncol(X))
  cov2<-100*diag(ncol(X))
  z<-rep(0,n)				# Latent Normal Variable


for (i in 1:nsim) {
  muz<-X%*%beta
  z[y==0]<-qnorm(runif(n,0,pnorm(0,muz,1)),muz,1)[y==0]
  z[y==1]<-qnorm(runif(n,pnorm(0,muz,1),1),muz,1)[y==1]
   mbeta <- vbeta%*%(prec0%*%mbeta0+crossprod(X,z))
   beta<-c(rmvnorm(1,mbeta,vbeta))
 
   if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
   Beta[j,,ch]<-beta
 
}

if (i%%1000==0)  print(c(ch, i))

}
}

 
# End of MCMC algorithm (Gibbs sampler)

## Convergence checks
library(coda)
BETA<-mcmc.list(as.mcmc(Beta[,,1]), as.mcmc(Beta[,,2]), 
		as.mcmc(Beta[,,3]))
gelman.diag(BETA)

Betas<-rbind(Beta[,,1], Beta[,,2], Beta[,,3])

Posterior.summaries<-cbind(colMeans(Betas), HPDinterval(as.mcmc(Betas)))
rownames(Posterior.summaries)<-colnames(X)
colnames(Posterior.summaries)<-c("Posterior Mean", "Posterior 95% HDP - low",
						"Posterior 95% HPD - high")
Posterior.summaries


# Posterior predictive comparisons

Posterior.Predictions<-sapply(1:nrow(Betas), function(s) ifelse(pnorm(X%*%Betas[s,])>=0.5,1,0))

Percent.Correct.Preditions<-mean(Posterior.Predictions==y)

Percent.Correct.Preditions
